import assert from 'assert';
import BN from 'bn.js';
import 'dotenv/config';
import bs58 from 'bs58';
import Big from 'big.js';

import * as anchor from "@coral-xyz/anchor";
import {
  TOKEN_PROGRAM_ID,
} from "@solana/spl-token";
import { Connection, Keypair, PublicKey } from "@solana/web3.js";

import { createVestingPlanV2 } from '../locker-utils';

assert(process.env.NEXT_PUBLIC_SOLANA_RPC_URL);
assert(process.env.WSOL_LOCKER_ACCOUNT_PK);
assert(process.env.WSOL_MINT_ADDRESS);

const RPC_ENDPOINT= process.env.NEXT_PUBLIC_SOLANA_RPC_URL;
const WSOL_MINT_ADDRESS = process.env.WSOL_MINT_ADDRESS;
const WSOL_LOCKER_ACCOUNT_PK = process.env.WSOL_LOCKER_ACCOUNT_PK;

const userKP = anchor.web3.Keypair.fromSecretKey(bs58.decode(WSOL_LOCKER_ACCOUNT_PK));

const connection = new Connection(RPC_ENDPOINT);
const token = new PublicKey(WSOL_MINT_ADDRESS);

const provider = new anchor.AnchorProvider(
  connection,
  new anchor.Wallet(userKP),
  // Commitment level required for simulating transaction
  { preflightCommitment: 'processed' }
);

anchor.setProvider(provider);

export async function createLock(tokenLockerKeypair: anchor.web3.Keypair, recipientPubKey: anchor.web3.PublicKey, duration: BN, amount: BN): Promise<anchor.web3.PublicKey | void> {

  if (amount.eq(new BN(0))) {
    console.log('Invalid Amount');
    return;
  }

  console.log('Creating a lock...');

  const escrow = await createVestingPlanV2({
    ownerKeypair: tokenLockerKeypair,
    vestingStartTime: new BN(Math.floor(Date.now() / 1000) - 60),  // Start immediately
    tokenMint: token,
    isAssertion: true,
    cliffTime: duration,
    frequency: new BN(1), // Not needed since full unlock happens at cliff
    cliffUnlockAmount: amount, // The entire amount should be released at cliff
    amountPerPeriod: new BN(0), // No tokens should be released before cliff
    numberOfPeriod: new BN(1), // Only release tokens once
    recipient: recipientPubKey,
    updateRecipientMode: 0,
    cancelMode: 1, // Only creator can cancel the lock
    tokenProgram: TOKEN_PROGRAM_ID,
  });

  if (escrow) {
    console.log('Lock created successfully:', escrow.toString());
  }

  return escrow;
}

export async function createRewardLock(recipient: string, txMtmAmount: string) {
  const { WSOL_LOCKER_ACCOUNT_PK, WSOL_LOCK_DURATION_IN_SECONDS, WSOL_MINT_ADDRESS, NEXT_PUBLIC_MTM_MINT_ADDRESS, REWARD_MULTIPLIER } = process.env;

  if (!WSOL_LOCKER_ACCOUNT_PK || !WSOL_LOCK_DURATION_IN_SECONDS || !WSOL_MINT_ADDRESS || !NEXT_PUBLIC_MTM_MINT_ADDRESS || !REWARD_MULTIPLIER) {
    throw new Error('Missing required environment variables for creating reward wSOL lock');
  }

  const lockDuration = new BN(WSOL_LOCK_DURATION_IN_SECONDS).add(new BN(Math.floor(Date.now() / 1000)));
  const tokenLockerKeypair = Keypair.fromSecretKey(bs58.decode(WSOL_LOCKER_ACCOUNT_PK));
  const recipientPublicKey = new PublicKey(recipient);

  const url = `https://api.jup.ag/price/v2?ids=${NEXT_PUBLIC_MTM_MINT_ADDRESS}&vsToken=${WSOL_MINT_ADDRESS}`;
  const response = await fetch(url);
  const { data } = await response.json();

  const priceWSOLFor1MTM = new Big(data[NEXT_PUBLIC_MTM_MINT_ADDRESS].price).toFixed(9);
  const mtmAmount = new Big(txMtmAmount).div(new Big(10).pow(6));
  const wsolAmount = new BN(new Big(mtmAmount).times(priceWSOLFor1MTM).times(new Big(10).pow(9)).times(REWARD_MULTIPLIER).toFixed(0));

  return createLock(tokenLockerKeypair, recipientPublicKey, lockDuration, wsolAmount);
}
