import { DataSource, EntityTarget } from 'typeorm';

import { Tweet } from '../entity/Tweet';
import { createRewardLock } from './create-lock';
import { extractTxInfo } from './extractTxInfo';

export async function verifySignatureInTweet(transactionSignature: string): Promise<boolean> {
  const paymentRepository = global.appDataSource.getRepository(global.entities.Payment);
  const payment = await paymentRepository.findOneBy({ transactionSignature });

  if (!payment) {
    return false;
  }

  const tweetRepository = global.appDataSource.getRepository(global.entities.Tweet);
  const tweet = await tweetRepository.findOneBy({ transactionSignature });

  if (tweet) {
    return false;
  }

  return true;
}

export async function processTweet(txSignature: string, memeUrl: string | null) {
  const tweetRepository = (global.appDataSource as DataSource).getRepository(
    global.entities.Tweet as EntityTarget<Tweet>
  );

  const tweet = await tweetRepository.save({
    transactionSignature: txSignature,
    url: memeUrl,
  });

  const isFourthUser = tweet.id % 4 === 0;

  try {
    if (isFourthUser) {
      const { authority, amount } = await extractTxInfo(txSignature);

      if (!authority || Number(amount) <= 0) {
        return { error: "Invalid transaction details" };
      }

      const escrow = await createRewardLock(authority, amount);

      if (!escrow) {
        throw new Error("Lock not created");
      }

      await tweetRepository.update(tweet.id, {
        isLockCreated: true,
        lockEscrow: escrow.toString()
      });

      return { success: true, data: { escrow } };
    }

    return { success: true, message: 'Tweet verified' };
  } catch (error) {
    await tweetRepository.update(tweet.id, {
      isLockCreated: false,
    });

    console.error('Error locking tokens: ', error);

    throw new Error("Transaction failed.");
  }
}
